#define GAME_DLL

#include "GMLuaModule.h"
#include "engine/ivmodelinfo.h"
#include "materialsystem/imaterial.h"
#include "cbase.h"

ILuaInterface*	g_Lua = NULL;
IVModelInfo*	g_ModelInfo = NULL;

/*------------------------------------------------------
	Purpose: push a vector object on to the stack
------------------------------------------------------*/
void PushVector( Vector& vector )
{
	
	g_Lua->Push( g_Lua->GetGlobal( "Vector" ) );
	g_Lua->Push( vector.x );
	g_Lua->Push( vector.y );
	g_Lua->Push( vector.z );
	g_Lua->Call( 3, 1 );
	
	// partially taken from renderx( ... )
	g_Lua->Push( g_Lua->GetReturn( 0 ) );
	
}

/*------------------------------------------------------
	Purpose: get the modelindex of a precached model
------------------------------------------------------*/
LUA_FUNCTION( GetIndex )
{
	
	const char* strName = g_Lua->GetStringOrError( 1 );
	float nIndex = float( g_ModelInfo->GetModelIndex( strName ) );
	
	g_Lua->Push( nIndex );
	
	return( 1 );
	
}

/*------------------------------------------------------
	Purpose: get a list of materials the model uses
	Note: this function crashes the game when used
	client side!
------------------------------------------------------*/
LUA_FUNCTION( GetMaterials )
{
	
	g_Lua->CheckType( 1, GLua::TYPE_NUMBER );
	
	const model_t* pModel = g_ModelInfo->GetModel( g_Lua->GetNumber( 1 ) );
	
	// GetModelMaterialCount always returns zero
	//int nMatCount = g_ModelInfo->GetModelMaterialCount( pModel );
	
	IMaterial* pMaterials;
	g_ModelInfo->GetModelMaterials( pModel, 1, &pMaterials );
	
	g_Lua->Push( pMaterials->GetName( ) );
	
	/*
	ILuaObject* pTable = g_Lua->GetNewTable( );
	
	for( int i = 0; i < nMatCount; i++ )
	{
		
		IMaterial* pMat = &pMaterials[ i ];
		
		pTable->SetMember( i + 1, pMat->GetName( ) );
		
	}
	
	g_Lua->Push( pTable );
	*/
	
	return( 1 );
	
}

/*------------------------------------------------------
	Purpose: get radius of a model
------------------------------------------------------*/
LUA_FUNCTION( GetRadius )
{
	
	g_Lua->CheckType( 1, GLua::TYPE_NUMBER );
	
	const model_t* pModel = g_ModelInfo->GetModel( g_Lua->GetNumber( 1 ) );
	
	g_Lua->Push( g_ModelInfo->GetModelRadius( pModel ) );
	
	return( 1 );
	
}

/*------------------------------------------------------
	Purpose: get bounds of a model
------------------------------------------------------*/
LUA_FUNCTION( GetBounds )
{
	
	g_Lua->CheckType( 1, GLua::TYPE_NUMBER );
	
	const model_t* pModel = g_ModelInfo->GetModel( g_Lua->GetNumber( 1 ) );
	
	Vector vecMin, vecMax;
	g_ModelInfo->GetModelBounds( pModel, vecMin, vecMax );
	
	PushVector( vecMin );
	PushVector( vecMax );
	
	return( 2 );
	
}

/*------------------------------------------------------
	Purpose: get render bounds of a model
------------------------------------------------------*/
LUA_FUNCTION( GetRenderBounds )
{
	
	g_Lua->CheckType( 1, GLua::TYPE_NUMBER );
	
	const model_t* pModel = g_ModelInfo->GetModel( g_Lua->GetNumber( 1 ) );
	
	Vector vecMin, vecMax;
	g_ModelInfo->GetModelRenderBounds( pModel, vecMin, vecMax );
	
	PushVector( vecMin );
	PushVector( vecMax );
	
	return( 2 );
	
}

GMOD_MODULE( Init, Shutdown );

/*------------------------------------------------------
	Purpose: initialize the module
------------------------------------------------------*/
LUA_FUNCTION( Init )
{
	
	g_Lua = Lua( );
	
	CreateInterfaceFn enginedll = Sys_GetFactory( "engine.dll" );
	
	if( g_Lua->IsClient( ) )
		g_ModelInfo = ( IVModelInfo* )enginedll( VMODELINFO_CLIENT_INTERFACE_VERSION, NULL );
	else if( g_Lua->IsServer( ) )
		g_ModelInfo = ( IVModelInfo* )enginedll( VMODELINFO_SERVER_INTERFACE_VERSION, NULL );
	
	if( !g_ModelInfo )
	{
		
		g_Lua->Error( "Failed to load interfaces..\n" );
		
		return( 1 );
		
	}
	else
	{
		
		ILuaObject* pTable = g_Lua->GetNewTable( );
		
		if( pTable )
		{
			
			pTable->SetMember( "GetIndex",		GetIndex );
			pTable->SetMember( "GetMaterials",		GetMaterials );
			pTable->SetMember( "GetRadius",		GetRadius );
			pTable->SetMember( "GetBounds",		GetBounds );
			pTable->SetMember( "GetRenderBounds",	GetRenderBounds );
			
		}
		
		g_Lua->SetGlobal( "modelinfo", pTable );
		
	}
	
	return( 0 );
	
}

/*------------------------------------------------------
	Purpose: shut down the module
------------------------------------------------------*/
LUA_FUNCTION( Shutdown )
{
	
	return( 0 );
	
}